@@ -629,11 +629,6 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 	g_free (main_message);
 	g_free (secondary_message);
 
-	/* update the infos */
-	brasero_burn_dialog_update_info (dialog,
-					 &priv->input, 
-					 brasero_burn_session_get_dest_media (priv->session));
-
 	if (result != GTK_RESPONSE_OK)
 		return BRASERO_BURN_CANCEL;
 
@@ -751,8 +746,7 @@ brasero_burn_dialog_image_error (BraseroBurn *burn,
 		format = brasero_burn_session_get_output_format (priv->session);
 		brasero_burn_session_get_output (priv->session,
 						 &image,
-						 &toc,
-						 NULL);
+						 &toc);
 
 		if (toc) {
 			gchar *name;
@@ -861,9 +855,10 @@ brasero_burn_dialog_data_loss_cb (BraseroBurn *burn,
 				  GtkDialog *dialog)
 {
 	return brasero_burn_dialog_loss_warnings_cb (dialog,
-						     _("Do you really want to erase the current disc?"),
+						     _("Do you really want to blank the current disc?"),
 						     _("The disc in the drive holds data."),
-						     _("_Erase Disc"),
+	                                             /* Translators: Blank is a verb here */
+						     _("_Blank Disc"),
 						     "media-optical-blank");
 }
 
@@ -895,7 +890,7 @@ brasero_burn_dialog_audio_to_appendable_cb (BraseroBurn *burn,
 	BraseroBurnResult result;
 
 	secondary = g_strdup_printf ("%s\n%s",
-				     _("You might not be able to listen to them with stereos and CD-TEXT won't be written."),
+				     _("CD-RW audio discs may not play correctly in older CD players and CD-TEXT won't be written."),
 				     _("Do you want to continue anyway?"));
 
 	result = brasero_burn_dialog_loss_warnings_cb (dialog,
@@ -915,7 +910,7 @@ brasero_burn_dialog_rewritable_cb (BraseroBurn *burn,
 	BraseroBurnResult result;
 
 	secondary = g_strdup_printf ("%s\n%s",
-				     _("You might not be able to listen to it with stereos."),
+				     _("CD-RW audio discs may not play correctly in older CD players."),
 				     _("Do you want to continue anyway?"));
 
 	result = brasero_burn_dialog_loss_warnings_cb (dialog,
@@ -1178,9 +1173,6 @@ brasero_burn_dialog_action_changed_cb (BraseroBurn *burn,
 		BraseroMedia media = BRASERO_MEDIUM_NONE;
 
 		brasero_burn_status (burn, &media, NULL, NULL, NULL);
-		brasero_burn_dialog_update_info (dialog,
-						 &priv->input,
-						 media);
 	}
 
 	priv->is_creating_image = (action == BRASERO_BURN_ACTION_CREATING_IMAGE);
@@ -1387,6 +1379,13 @@ brasero_burn_dialog_finalize (GObject * object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * brasero_burn_dialog_new:
+ *
+ * Creates a new #BraseroBurnDialog object
+ *
+ * Return value: a #GtkWidget. Unref when it is not needed anymore.
+ **/
 GtkWidget *
 brasero_burn_dialog_new (void)
 {
@@ -1901,7 +1900,11 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	brasero_burn_dialog_activity_stop (dialog, primary);
 	g_free (primary);
 
-	if (!brasero_burn_session_is_dest_file (priv->session)) {
+	/* Don't show the "Make Another Copy" button if:
+	 * - we wrote to a file
+	 * - we wrote a merged session */
+	if (!brasero_burn_session_is_dest_file (priv->session)
+	&&!(brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_MERGE)) {
 		/* Useful button but it shouldn't be used for images */
 		make_another = gtk_dialog_add_button (GTK_DIALOG (dialog),
 						      _("Make _Another Copy"),
@@ -2162,6 +2165,7 @@ brasero_burn_dialog_record_session (BraseroBurnDialog *dialog,
 
 	priv = BRASERO_BURN_DIALOG_PRIVATE (dialog);
 
+	/* Update info */
 	brasero_burn_dialog_update_info (dialog, &priv->input, media);
 
 	/* start the recording session */
@@ -2227,6 +2231,15 @@ brasero_burn_dialog_wait_for_ready_state (BraseroBurnDialog *dialog)
 	return (result == BRASERO_BURN_OK);
 }
 
+/**
+ * brasero_burn_dialog_run:
+ * @dialog: a #BraseroBurnDialog
+ * @session: a #BraseroBurnSession
+ *
+ * Start burning the contents of @session.
+ *
+ * Return value: a #gboolean. TRUE if the operation was successfully carried out, FALSE otherwise.
+ **/
 gboolean
 brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 			 BraseroBurnSession *session)
@@ -2262,12 +2275,19 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 	}
 
 	do {
+		if (!GTK_WIDGET_VISIBLE (dialog))
+			gtk_widget_show (GTK_WIDGET (dialog));
+
 		result = brasero_burn_dialog_record_session (dialog, media);
 	} while (result == BRASERO_BURN_RETRY);
 
 	priv->session = NULL;
 	g_object_unref (session);
 
+	/* re-enable autoconfiguration */
+	if (BRASERO_IS_SESSION_CFG (priv->session))
+		brasero_session_cfg_enable (BRASERO_SESSION_CFG (priv->session));
+
 	return (result == BRASERO_BURN_OK);
 }
 
@@ -2309,6 +2329,16 @@ brasero_burn_dialog_cancel_dialog (GtkWidget *toplevel)
 	return (result != GTK_RESPONSE_OK);
 }
 
+/**
+ * brasero_burn_dialog_cancel:
+ * @dialog: a #BraseroBurnDialog
+ * @force_cancellation: a #gboolean
+ *
+ * Cancel the ongoing operation run by @dialog; if @force_cancellation is FALSE then it can
+ * happen that the operation won't be cancelled if there is a risk to make a disc unusable.
+ *
+ * Return value: a #gboolean. TRUE if it was sucessfully cancelled, FALSE otherwise.
+ **/
 gboolean
 brasero_burn_dialog_cancel (BraseroBurnDialog *dialog,
 			    gboolean force_cancellation)
