@@ -318,9 +318,20 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 	}
 
 	if (valid == BRASERO_SESSION_INSUFFICIENT_SPACE) {
+		goffset min_disc_size;
+		goffset available_space;
+
+		min_disc_size = brasero_session_span_get_max_space (BRASERO_SESSION_SPAN (priv->session));
+
+		/* One rule should be that the maximum batch size should not exceed the disc size
+		 * FIXME: we could change it into a dialog telling the user what is the maximum
+		 * size required. */
+		available_space = brasero_burn_session_get_available_medium_space (BRASERO_BURN_SESSION (priv->session));
+
 		/* Here there is an alternative: we may be able to span the data
 		 * across multiple media. So try that. */
-		if (brasero_session_span_possible (BRASERO_SESSION_SPAN (priv->session)) == BRASERO_BURN_RETRY) {
+		if (available_space > min_disc_size
+		&& brasero_session_span_possible (BRASERO_SESSION_SPAN (priv->session)) == BRASERO_BURN_RETRY) {
 			GtkWidget *message;
 
 			message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
@@ -653,6 +664,7 @@ brasero_burn_options_setup_audio (BraseroBurnOptions *self)
 static void
 brasero_burn_options_setup_video (BraseroBurnOptions *self)
 {
+	gchar *title;
 	GtkWidget *options;
 	BraseroBurnOptionsPrivate *priv;
 
@@ -671,6 +683,14 @@ brasero_burn_options_setup_video (BraseroBurnOptions *self)
 
 	/* create the options box */
 	options = brasero_video_options_new (BRASERO_BURN_SESSION (priv->session));
+	gtk_widget_show (options);
+
+	title = g_strdup_printf ("<b>%s</b>", _("Video Options"));
+	options = brasero_utils_pack_properties (title,
+	                                           options,
+	                                           NULL);
+	g_free (title);
+
 	gtk_widget_show (options);
 	brasero_burn_options_add_options (self, options);
 }
@@ -878,7 +898,7 @@ brasero_burn_options_setup (BraseroBurnOptions *self)
 			brasero_burn_options_setup_data (self);
 	}
 	else if (brasero_track_type_get_has_stream (type)) {
-		if (brasero_track_type_get_stream_format (type) & (BRASERO_VIDEO_FORMAT_UNDEFINED|BRASERO_VIDEO_FORMAT_VCD|BRASERO_VIDEO_FORMAT_VIDEO_DVD)) {
+		if (BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type))) {
 			if (!priv->has_video)
 				brasero_burn_options_setup_video (self);
 		}
@@ -905,6 +925,14 @@ brasero_burn_options_track_removed (BraseroBurnSession *session,
 	brasero_burn_options_setup (dialog);
 }
 
+static void
+brasero_burn_options_track_changed (BraseroBurnSession *session,
+                                    BraseroTrack *track,
+                                    BraseroBurnOptions *dialog)
+{
+	brasero_burn_options_setup (dialog);
+}
+
 static void
 brasero_burn_options_set_property (GObject *object,
 				   guint prop_id,
@@ -932,6 +960,10 @@ brasero_burn_options_set_property (GObject *object,
 				  "track-removed",
 				  G_CALLBACK (brasero_burn_options_track_removed),
 				  object);
+		g_signal_connect (priv->session,
+				  "track-changed",
+				  G_CALLBACK (brasero_burn_options_track_changed),
+				  object);
 		brasero_burn_options_build_contents (BRASERO_BURN_OPTIONS (object));
 		brasero_burn_options_setup (BRASERO_BURN_OPTIONS (object));
 
@@ -993,6 +1025,9 @@ brasero_burn_options_finalize (GObject *object)
 		g_signal_handlers_disconnect_by_func (priv->session,
 						      brasero_burn_options_track_removed,
 						      object);
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_burn_options_track_changed,
+						      object);
 		g_signal_handlers_disconnect_by_func (priv->session,
 						      brasero_burn_options_valid_cb,
 						      object);
