@@ -145,6 +145,8 @@ static guint brasero_burn_signals [LAST_SIGNAL] = { 0 };
 
 #define MAX_EJECT_ATTEMPTS	5
 #define MAX_MOUNT_ATTEMPTS	40
+#define MAX_REPROBE_ATTEMPTS	80
+
 #define MOUNT_TIMEOUT		500
 
 static GObjectClass *parent_class = NULL;
@@ -161,6 +163,14 @@ brasero_burn_powermanagement (BraseroBurn *self,
 		brasero_uninhibit_suspend (priv->appcookie); 
 }
 
+/**
+ * brasero_burn_new:
+ *
+ *  Creates a new #BraseroBurn object.
+ *
+ * Return value: a #BraseroBurn object.
+ **/
+
 BraseroBurn *
 brasero_burn_new ()
 {
@@ -249,16 +259,21 @@ brasero_burn_sleep (BraseroBurn *burn, gint msec)
 static BraseroBurnResult
 brasero_burn_reprobe (BraseroBurn *burn)
 {
+	guint attempts = 0;
 	BraseroMedium *medium;
 	BraseroBurnPrivate *priv;
 	BraseroBurnResult result = BRASERO_BURN_OK;
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
+	BRASERO_BURN_LOG ("Reprobing for medium");
+
 	/* reprobe the medium and wait for it to be probed */
 	brasero_drive_reprobe (priv->dest);
-	while (!(medium = brasero_drive_get_medium (priv->dest)))
+	while (attempts < MAX_REPROBE_ATTEMPTS && (medium = brasero_drive_get_medium (priv->dest)) == NULL) {
 		result = brasero_burn_sleep (burn, 250);
+		attempts ++;
+	}
 
 	return result;
 }
@@ -270,7 +285,7 @@ brasero_burn_eject (BraseroBurn *self,
 {
 	guint counter = 0;
 
-	brasero_drive_eject (drive, TRUE, error);
+	brasero_drive_eject (drive, TRUE, NULL);
 
 	/* sleep some time and see what happened */
 	brasero_burn_sleep (self, 500);
@@ -297,7 +312,7 @@ brasero_burn_eject (BraseroBurn *self,
 		}
 
 		BRASERO_BURN_LOG ("Retrying ejection");
-		brasero_drive_eject (drive, TRUE, error);
+		brasero_drive_eject (drive, TRUE, NULL);
 		brasero_burn_sleep (self, 500);
 	}
 
@@ -534,7 +549,7 @@ static BraseroBurnResult
 brasero_burn_lock_src_media (BraseroBurn *burn,
 			     GError **error)
 {
-	gchar *failure;
+	gchar *failure = NULL;
 	BraseroMedia media;
 	BraseroMedium *medium;
 	BraseroBurnResult result;
@@ -857,6 +872,29 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 	brasero_burn_session_get_input_type (priv->session, input);
 	flags = brasero_burn_session_get_flags (priv->session);
 
+	if (brasero_track_type_get_has_image (input)) {
+		goffset medium_sec = 0;
+		goffset session_sec = 0;
+
+		/* This test is only valid when it's an image
+		 * as input which comes handy when copying
+		 * from the same drive as we're sure we do 
+		 * have an image. */
+		brasero_medium_get_capacity (medium,
+		                             NULL,
+		                             &medium_sec);
+		brasero_burn_session_get_size (priv->session,
+		                               &session_sec,
+		                               NULL);
+
+		if (session_sec > medium_sec) {
+			BRASERO_BURN_LOG ("Not enough space for image %"G_GOFFSET_FORMAT"/%"G_GOFFSET_FORMAT);
+			berror = BRASERO_BURN_ERROR_MEDIUM_SPACE;
+			result = BRASERO_BURN_NEED_RELOAD;
+			goto end;
+		}
+	}
+
 	if (must_blank) {
 		/* There is an error if APPEND was set since this disc is not
 		 * supported without a prior blanking. */
@@ -1111,17 +1149,8 @@ brasero_burn_unlock_dest_media (BraseroBurn *burn,
 	brasero_drive_unlock (priv->dest);
 
 	if (!BRASERO_BURN_SESSION_EJECT (priv->session)) {
-		if (priv->dest) {
-			GDrive *gdrive;
-
-			gdrive = brasero_drive_get_gdrive (priv->dest);
-
-			/* reprobe the contents of the drive system wide */
-			g_drive_poll_for_media (gdrive, NULL, NULL, NULL);
-			g_object_unref (gdrive);
-
+		if (priv->dest)
 			brasero_drive_reprobe (priv->dest);
-		}
 	}
 	else
 		brasero_drive_eject (priv->dest, FALSE, error);
@@ -1188,6 +1217,20 @@ brasero_burn_action_changed (BraseroTask *task,
 	brasero_burn_action_changed_real (burn, action);
 }
 
+/**
+ * brasero_burn_get_action_string:
+ * @burn: a #BraseroBurn
+ * @action: a #BraseroBurnAction
+ * @string: a #gchar **
+ *
+ * This function returns the current action (in @string)  of
+ * an ongoing operation performed by @burn.
+ * @action is used to set a default string in case there was
+ * no string set by the backend to describe the current
+ * operation.
+ *
+ **/
+
 void
 brasero_burn_get_action_string (BraseroBurn *burn,
 				BraseroBurnAction action,
@@ -1207,6 +1250,24 @@ brasero_burn_get_action_string (BraseroBurn *burn,
 							    string);
 }
 
+/**
+ * brasero_burn_status:
+ * @burn: a #BraseroBurn
+ * @media: a #BraseroMedia or NULL
+ * @isosize: a #goffset or NULL
+ * @written: a #goffset or NULL
+ * @rate: a #guint64 or NULL
+ *
+ * Returns various information about the current operation 
+ * in @media (the current media type being burnt),
+ * @isosize (the size of the data being burnt), @written (the
+ * number of bytes having been written so far) and @rate
+ * (the speed at which data are written).
+ *
+ * Return value: a #BraseroBurnResult. BRASERO_BURN_OK if there is
+ * an ongoing operation; BRASERO_BURN_NOT_READY otherwise.
+ **/
+
 BraseroBurnResult
 brasero_burn_status (BraseroBurn *burn,
 		     BraseroMedia *media,
@@ -1399,8 +1460,7 @@ start:
 		 * would clutter the disk, wasting space. */
 		brasero_burn_session_get_output (priv->session,
 						 &image,
-						 &toc,
-						 NULL);
+						 &toc);
 		if (image)
 			g_remove (image);
 		if (toc)
@@ -1515,16 +1575,6 @@ brasero_burn_run_recorder (BraseroBurn *burn, GError **error)
 	burner = brasero_burn_session_get_burner (priv->session);
 	burnt_medium = brasero_drive_get_medium (burner);
 
-	/* before we start let's see if that drive can be used exclusively.
-	 * Of course, it's not really safe since a process could take a lock
-	 * just after us but at least it'll give some time to HAL and friends
-	 * to finish what they're doing. 
-	 * This was done because more than often backends wouldn't be able to 
-	 * get a lock on a medium after a simulation. */
-	result = brasero_burn_can_use_drive_exclusively (burn, burner);
-	if (result != BRASERO_BURN_OK)
-		return result;
-
 start:
 
 	/* this is just in case */
@@ -1552,6 +1602,16 @@ start:
 		return BRASERO_BURN_ERR;
 	}
 
+	/* before we start let's see if that drive can be used exclusively.
+	 * Of course, it's not really safe since a process could take a lock
+	 * just after us but at least it'll give some time to HAL and friends
+	 * to finish what they're doing. 
+	 * This was done because more than often backends wouldn't be able to 
+	 * get a lock on a medium after a simulation. */
+	result = brasero_burn_can_use_drive_exclusively (burn, burner);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
 	/* actual running of task */
 	result = brasero_task_run (priv->task, &ret_error);
 
@@ -1700,10 +1760,125 @@ start:
 	return BRASERO_BURN_ERR;
 }
 
-/* FIXME: for the moment we don't allow for mixed CD type */
+static BraseroBurnResult
+brasero_burn_check_session_consistency (BraseroBurn *burn,
+					GError **error)
+{
+	BraseroMedia media;
+	BraseroBurnFlag flag;
+	BraseroBurnFlag flags;
+	BraseroBurnFlag retval;
+	BraseroBurnResult result;
+	BraseroTrackType *type = NULL;
+	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
+	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
+	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+
+	BRASERO_BURN_DEBUG (burn, "Checking session consistency");
+
+	/* make sure there is a track in the session. */
+	type = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (priv->session, type);
+
+	if (brasero_track_type_is_empty (type)
+	|| !brasero_burn_session_get_tracks (priv->session)) {
+		brasero_track_type_free (type);
+
+		BRASERO_BURN_DEBUG (burn, "No track set");
+		g_set_error (error,
+			     BRASERO_BURN_ERROR,
+			     BRASERO_BURN_ERROR_GENERAL,
+			     _("There is no track to be burnt"));
+		return BRASERO_BURN_ERR;
+	}
+	brasero_track_type_free (type);
+
+	/* make sure there is a drive set as burner. */
+	if (!brasero_burn_session_is_dest_file (priv->session)) {
+		BraseroDrive *burner;
+
+		burner = brasero_burn_session_get_burner (priv->session);
+		if (!burner) {
+			BRASERO_BURN_DEBUG (burn, "No burner specified.");
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_OUTPUT_NONE,
+				     _("No burner specified"));
+			return BRASERO_BURN_ERR;	
+		}
+	}
+
+	media = brasero_burn_session_get_dest_media (priv->session);
+
+	/* save then wipe out flags from session to check them one by one */
+	flags = brasero_burn_session_get_flags (priv->session);
+	brasero_burn_session_set_flags (BRASERO_BURN_SESSION (priv->session), BRASERO_BURN_FLAG_NONE);
+
+	result = brasero_burn_session_get_burn_flags (priv->session,
+						      &supported,
+						      &compulsory);
+
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	for (flag = 1; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
+		/* see if this flag was originally set */
+		if (!(flags & flag))
+			continue;
+
+		/* Check each flag before re-adding it. Emit warnings to user
+		 * to know if he wants to carry on for some flags when they are
+		 * not supported; namely DUMMY. Other flags trigger an error.
+		 * No need for BURNPROOF since that usually means it is just the
+		 * media type that doesn't need it. */
+		if (supported & flag) {
+			brasero_burn_session_add_flag (priv->session, flag);
+			brasero_burn_session_get_burn_flags (priv->session,
+							     &supported,
+							     &compulsory);
+		}
+		else {
+			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported:");
+
+			if (flag & BRASERO_BURN_FLAG_DUMMY) {
+				/* This is simply a warning that it's not possible */
+
+			}
+			else if (flag & BRASERO_BURN_FLAG_MERGE) {
+				/* we pay attention to one flag in particular
+				 * (MERGE) if it was set then it must be
+				 * supported. Otherwise error out. */
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("Merging data is impossible with this disc"));
+				return BRASERO_BURN_ERR;
+			}
+			/* No need to tell the user burnproof is not supported
+			 * as these drives handle errors differently which makes
+			 * burnproof useless for them. */
+		}
+	}
+
+	retval = brasero_burn_session_get_flags (priv->session);
+	if (retval != flags)
+		BRASERO_BURN_LOG_FLAGS (retval, "Some flags were not supported. Corrected to");
+
+	if (retval != (retval | compulsory)) {
+		retval |= compulsory;
+		BRASERO_BURN_LOG_FLAGS (retval, "Some compulsory flags were forgotten. Corrected to");
+	}
+
+	brasero_burn_session_set_flags (priv->session, retval);
+	BRASERO_BURN_LOG_FLAGS (retval, "Flags after checking =");
+	return BRASERO_BURN_OK;
+}
+
+/* FIXME: at the moment we don't allow for mixed CD type */
 static BraseroBurnResult
 brasero_burn_run_tasks (BraseroBurn *burn,
 			gboolean erase_allowed,
+                        gboolean *dummy_session,
 			GError **error)
 {
 	BraseroBurnResult result;
@@ -1720,8 +1895,21 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 	priv->task_nb = g_slist_length (tasks);
 	BRASERO_BURN_LOG ("%i tasks to perform", priv->task_nb);
 
+	/* push the session settings to keep the original session untainted */
+	brasero_burn_session_push_settings (priv->session);
+
+	/* check flags consistency */
+	result = brasero_burn_check_session_consistency (burn, error);
+	if (result != BRASERO_BURN_OK) {
+		brasero_burn_session_pop_settings (priv->session);
+		return result;
+	}
+
 	/* run all imaging tasks first */
 	for (iter = tasks; iter; iter = next) {
+		goffset len = 0;
+		BraseroDrive *drive;
+		BraseroMedium *medium;
 		BraseroTaskAction action;
 
 		next = iter->next;
@@ -1737,9 +1925,19 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 				  G_CALLBACK (brasero_burn_action_changed),
 				  burn);
 
-		/* see what type of task it is. It could be a blank/erase one */
+		/* see what type of task it is. It could be a blank/erase one. */
+		/* FIXME!
+		 * If so then that's time to test the size of the image against
+		 * the size of the disc since erasing/formatting is always left
+		 * for the end, just before burning. We would not like to 
+		 * blank a disc and tell the user right after that the size of
+		 * the disc is not enough. */
 		action = brasero_task_ctx_get_action (BRASERO_TASK_CTX (priv->task));
 		if (action == BRASERO_TASK_ACTION_ERASE) {
+			/* FIXME: how could it be possible for a drive to test
+			 * with a CLOSED CDRW for example. Maybe we should
+			 * format/blank anyway. */
+
 			/* This is to avoid a potential problem when running a 
 			 * dummy session first. When running dummy session the 
 			 * media gets erased if need be. Since it is not
@@ -1749,6 +1947,31 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 				result = brasero_burn_run_eraser (burn, error);
 				if (result != BRASERO_BURN_OK)
 					break;
+
+				/* Reprobe. It can happen (like with dvd+rw-format) that
+				 * for the whole OS, the disc doesn't exist during the 
+				 * formatting. Wait for the disc to reappear */
+				/*  Likewise, this is necessary when we do a
+				 * simulation before blanking since it blanked the disc
+				 * and then to create all tasks necessary for the real
+				 * burning operation, we'll need the real medium status 
+				 * not to include a blanking job again. */
+				result = brasero_burn_reprobe (burn);
+				if (result != BRASERO_BURN_OK)
+					break;
+
+				/* Since we blanked/formatted we need to recheck the burn 
+				 * flags with the new medium type as some flags could have
+				 * been given the benefit of the double (MULTI with a CLOSED
+				 * CD for example). Recheck the original flags as they were
+				 * passed. */
+				/* FIXME: for some important flags we should warn the user
+				 * that it won't be possible */
+				brasero_burn_session_pop_settings (priv->session);
+				brasero_burn_session_push_settings (priv->session);
+				result = brasero_burn_check_session_consistency (burn, error);
+				if (result != BRASERO_BURN_OK)
+					break;
 			}
 			else
 				result = BRASERO_BURN_OK;
@@ -1757,18 +1980,6 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 			priv->task = NULL;
 			priv->tasks_done ++;
 
-			/* Reprobe. It can happen (like with dvd+rw-format) that
-			 * for the whole OS, the disc doesn't exist during the 
-			 * formatting. Wait for the disc to reappear */
-			/*  Likewise, this is necessary when we do a
-			 * simulation before blanking since it blanked the disc
-			 * and then to create all tasks necessary for the real
-			 * burning operation, we'll need the real medium status 
-			 * not to include a blanking job again. */
-			result = brasero_burn_reprobe (burn);
-			if (result != BRASERO_BURN_OK)
-				break;
-
 			continue;
 		}
 
@@ -1781,36 +1992,32 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 			break;
 
 		/* try to get the output size */
-		if (BRASERO_MEDIUM_RANDOM_WRITABLE (brasero_burn_session_get_dest_media (priv->session))) {
-			goffset len = 0;
-			BraseroDrive *drive;
-			BraseroMedium *medium;
+		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+							  &len,
+							  NULL);
 
-			brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
-								  &len,
-								  NULL);
+		drive = brasero_burn_session_get_burner (priv->session);
+		medium = brasero_drive_get_medium (drive);
 
-			drive = brasero_burn_session_get_burner (priv->session);
-			medium = brasero_drive_get_medium (drive);
+		if (brasero_burn_session_get_flags (priv->session) & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
+			priv->session_start = brasero_medium_get_next_writable_address (medium);
+		else
+			priv->session_start = 0;
 
-			if (brasero_burn_session_get_flags (priv->session) & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
-				priv->session_start = brasero_medium_get_next_writable_address (medium);
-			else
-				priv->session_start = 0;
+		priv->session_end = priv->session_start + len;
 
-			priv->session_end = priv->session_start + len;
-
-			BRASERO_BURN_LOG ("Burning from %lld to %lld",
-					  priv->session_start,
-					  priv->session_end);
-		}
+		BRASERO_BURN_LOG ("Burning from %lld to %lld",
+				  priv->session_start,
+				  priv->session_end);
 
 		/* see if we reached a recording task: it's the last task */
 		if (!next) {
-			if (brasero_burn_session_is_dest_file (priv->session))
-				result = brasero_burn_run_imager (burn, FALSE, error);
-			else
+			if (!brasero_burn_session_is_dest_file (priv->session)) {
+				*dummy_session = (brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_DUMMY);
 				result = brasero_burn_run_recorder (burn, error);
+			}
+			else
+				result = brasero_burn_run_imager (burn, FALSE, error);
 
 			if (result == BRASERO_BURN_OK)
 				priv->tasks_done ++;
@@ -1828,6 +2035,11 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 		priv->tasks_done ++;
 	}
 
+	/* restore the session settings. Keep the used flags
+	 * nevertheless to make sure we actually use the flags that were
+	 * set after checking for session consistency. */
+	brasero_burn_session_pop_settings (priv->session);
+
 	if (priv->task) {
 		g_object_unref (priv->task);
 		priv->task = NULL;
@@ -1915,136 +2127,27 @@ brasero_burn_check_real (BraseroBurn *self,
 	return result;
 }
 
-static BraseroBurnResult
-brasero_burn_check_session_consistency (BraseroBurn *burn,
-					GError **error)
-{
-	BraseroMedia media;
-	BraseroBurnFlag flag;
-	BraseroBurnFlag flags;
-	BraseroBurnFlag retval;
-	BraseroBurnResult result;
-	BraseroTrackType *type = NULL;
-	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
-
-	BRASERO_BURN_DEBUG (burn, "Checking session consistency");
-
-	/* make sure there is a track in the session. */
-	type = brasero_track_type_new ();
-	brasero_burn_session_get_input_type (priv->session, type);
-
-	if (brasero_track_type_is_empty (type)
-	|| !brasero_burn_session_get_tracks (priv->session)) {
-		brasero_track_type_free (type);
-
-		BRASERO_BURN_DEBUG (burn, "No track set");
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("There is no track to be burnt"));
-		return BRASERO_BURN_ERR;
-	}
-	brasero_track_type_free (type);
-
-	/* make sure there is a drive set as burner. */
-	if (!brasero_burn_session_is_dest_file (priv->session)) {
-		BraseroDrive *burner;
-
-		burner = brasero_burn_session_get_burner (priv->session);
-		if (!burner) {
-			BRASERO_BURN_DEBUG (burn, "No burner specified.");
-			g_set_error (error,
-				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_OUTPUT_NONE,
-				     _("No burner specified"));
-			return BRASERO_BURN_ERR;	
-		}
-	}
-
-	media = brasero_burn_session_get_dest_media (priv->session);
-
-	/* save then wipe out flags from session to check them one by one */
-	flags = brasero_burn_session_get_flags (priv->session);
-	brasero_burn_session_remove_flag (priv->session, flags);
-
-	result = brasero_burn_session_get_burn_flags (priv->session,
-						      &supported,
-						      &compulsory);
-	if (result != BRASERO_BURN_OK)
-		return result;
-
-	for (flag = 1; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
-		/* see if this flag was originally set */
-		if (!(flags & flag))
-			continue;
-
-		/* Check each flag before re-adding it. Emit warnings to user
-		 * to know if he wants to carry on for some flags when they are
-		 * not supported; namely DUMMY. Other flags trigger an error.
-		 * No need for BURNPROOF since that usually means it is just the
-		 * media type that doesn't need it. */
-		if (supported & flag) {
-			brasero_burn_session_add_flag (priv->session, flag);
-			brasero_burn_session_get_burn_flags (priv->session,
-							     &supported,
-							     &compulsory);
-		}
-		else {
-			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported:");
-
-			if (flag & BRASERO_BURN_FLAG_DUMMY) {
-				/* This is simply a warning that it's not possible */
-
-			}
-			else if (flag & BRASERO_BURN_FLAG_MERGE) {
-				/* we pay attention to one flag in particular
-				 * (MERGE) if it was set then it must be
-				 * supported. Otherwise error out. */
-				g_set_error (error,
-					     BRASERO_BURN_ERROR,
-					     BRASERO_BURN_ERROR_GENERAL,
-					     _("Merging data is impossible with this disc"));
-				return BRASERO_BURN_ERR;
-			}
-			/* No need to tell the user burnproof is not supported
-			 * as these drives handle errors differently which makes
-			 * burnproof useless for them. */
-		}
-	}
-
-	retval = brasero_burn_session_get_flags (priv->session);
-	if (retval != flags)
-		BRASERO_BURN_LOG_FLAGS (retval, "Some flags were not supported. Corrected to");
-
-	if (retval != (retval | compulsory)) {
-		retval |= compulsory;
-		BRASERO_BURN_LOG_FLAGS (retval, "Some compulsory flags were forgotten. Corrected to");
-	}
-
-	brasero_burn_session_set_flags (priv->session, retval);
-	BRASERO_BURN_LOG_FLAGS (retval, "Flags after checking =");
-	return BRASERO_BURN_OK;
-}
-
 static void
 brasero_burn_unset_checksums (BraseroBurn *self)
 {
 	GSList *tracks;
+	BraseroTrackType *type;
 	BraseroBurnPrivate *priv;
 
 	priv = BRASERO_BURN_PRIVATE (self);
 
 	tracks = brasero_burn_session_get_tracks (priv->session);
+	type = brasero_track_type_new ();
 	for (; tracks; tracks = tracks->next) {
 		BraseroTrack *track;
 
-		/* unset checksum (might depend from copy to another). */
 		track = tracks->data;
-		brasero_track_set_checksum (track,
-					    BRASERO_CHECKSUM_NONE,
-					    NULL);
+		brasero_track_get_track_type (track, type);
+		if (!brasero_track_type_get_has_image (type)
+		&& !brasero_track_type_get_has_medium (type))
+			brasero_track_set_checksum (track,
+						    BRASERO_CHECKSUM_NONE,
+						    NULL);
 	}
 }
 
@@ -2053,33 +2156,23 @@ brasero_burn_record_session (BraseroBurn *burn,
 			     gboolean erase_allowed,
 			     GError **error)
 {
-	BraseroBurnFlag session_flags;
+	const gchar *checksum = NULL;
 	BraseroTrack *track = NULL;
 	BraseroChecksumType type;
 	BraseroBurnPrivate *priv;
 	BraseroBurnResult result;
 	GError *ret_error = NULL;
 	BraseroMedium *medium;
+	gboolean dummy_session;
 	GSList *tracks;
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
-	/* unset checksum since no image has the exact same even if it is 
-	 * created from the same files */
+	/* unset checksum since no image has the exact
+	 * same even if it is created from the same files */
 	brasero_burn_unset_checksums (burn);
 
-	session_flags = BRASERO_BURN_FLAG_NONE;
 	do {
-		/* push the session settings to keep the original session untainted */
-		brasero_burn_session_push_settings (priv->session);
-
-		/* check flags consistency */
-		result = brasero_burn_check_session_consistency (burn, error);
-		if (result != BRASERO_BURN_OK) {
-			brasero_burn_session_pop_settings (priv->session);
-			break;
-		}
-
 		if (ret_error) {
 			g_error_free (ret_error);
 			ret_error = NULL;
@@ -2087,13 +2180,8 @@ brasero_burn_record_session (BraseroBurn *burn,
 
 		result = brasero_burn_run_tasks (burn,
 						 erase_allowed,
+		                                 &dummy_session,
 						 &ret_error);
-
-		/* restore the session settings. Keep the used flags
-		 * nevertheless to make sure we actually use the flags that were
-		 * set after checking for session consistency. */
-		session_flags = brasero_burn_session_get_flags (priv->session);
-		brasero_burn_session_pop_settings (priv->session);
 	} while (result == BRASERO_BURN_RETRY);
 
 	if (result != BRASERO_BURN_OK) {
@@ -2112,7 +2200,7 @@ brasero_burn_record_session (BraseroBurn *burn,
 	if (brasero_burn_session_is_dest_file (priv->session))
 		return BRASERO_BURN_OK;
 
-	if (session_flags & BRASERO_BURN_FLAG_DUMMY) {
+	if (dummy_session) {
 		/* if we are in dummy mode and successfully completed then:
 		 * - no need to checksum the media afterward (done later)
 		 * - no eject to have automatic real burning */
@@ -2149,50 +2237,38 @@ brasero_burn_record_session (BraseroBurn *burn,
 
 	track = tracks->data;
 	type = brasero_track_get_checksum_type (track);
-	if (type == BRASERO_CHECKSUM_NONE)
-		return BRASERO_BURN_OK;
-
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
-	||  type == BRASERO_CHECKSUM_SHA256) {
-		const gchar *checksum = NULL;
-
+	||  type == BRASERO_CHECKSUM_SHA256)
 		checksum = brasero_track_get_checksum (track);
+	else if (type == BRASERO_CHECKSUM_MD5_FILE)
+		checksum = BRASERO_MD5_FILE;
+	else if (type == BRASERO_CHECKSUM_SHA1_FILE)
+		checksum = BRASERO_SHA1_FILE;
+	else if (type == BRASERO_CHECKSUM_SHA256_FILE)
+		checksum = BRASERO_SHA256_FILE;
+	else
+		return BRASERO_BURN_OK;
 
-		/* the idea is to push a new track on the stack with
-		 * the current disc burnt and the checksum generated
-		 * during the session recording */
-		track = BRASERO_TRACK (brasero_track_disc_new ());
-		brasero_track_set_checksum (BRASERO_TRACK (track), type, checksum);
-	}
-	else if (type == BRASERO_CHECKSUM_MD5_FILE) {
-		track = BRASERO_TRACK (brasero_track_disc_new ());
-		brasero_track_set_checksum (BRASERO_TRACK (track),
-					    type,
-					    BRASERO_MD5_FILE);
-	}
-	else if (type == BRASERO_CHECKSUM_SHA1_FILE) {
-		track = BRASERO_TRACK (brasero_track_disc_new ());
-		brasero_track_set_checksum (BRASERO_TRACK (track),
-					    type,
-					    BRASERO_SHA1_FILE);
-	}
-	else if (type == BRASERO_CHECKSUM_SHA256_FILE) {
-		track = BRASERO_TRACK (brasero_track_disc_new ());
-		brasero_track_set_checksum (BRASERO_TRACK (track),
-					    type,
-					    BRASERO_SHA256_FILE);
-	}
-
+	/* the idea is to push a new track on the stack with
+	 * the current disc burnt and the checksum generated
+	 * during the session recording */
 	brasero_burn_session_push_tracks (priv->session);
 
+	track = BRASERO_TRACK (brasero_track_disc_new ());
+	brasero_track_set_checksum (BRASERO_TRACK (track),
+	                            type,
+	                            checksum);
+
 	brasero_track_disc_set_drive (BRASERO_TRACK_DISC (track), brasero_burn_session_get_burner (priv->session));
-	brasero_burn_session_add_track (priv->session, track);
+	brasero_burn_session_add_track (priv->session, track, NULL);
 
 	/* It's good practice to unref the track afterwards as we don't need it
 	 * anymore. BraseroBurnSession refs it. */
 	g_object_unref (track);
 
+	BRASERO_BURN_DEBUG (burn, "Preparing to checksum (type %i %s)", type, checksum);
+
 	/* this may be necessary for the drive to settle down and possibly be
 	 * mounted by gnome-volume-manager (just temporarily) */
 	result = brasero_burn_sleep (burn, 5000);
@@ -2210,45 +2286,37 @@ brasero_burn_record_session (BraseroBurn *burn,
 
 	medium = brasero_drive_get_medium (priv->dest);
 
+	/* Why do we do this?
+	 * Because for a lot of medium types the size
+	 * of the track return is not the real size of the
+	 * data that was written; examples
+	 * - CD that was written in SAO mode 
+	 * - a DVD-R which usually aligns its track size
+	 *   to a 16 block boundary
+	 */
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
 	||  type == BRASERO_CHECKSUM_SHA256) {
-		BraseroMedia media;
+		GValue *value;
 
-		/* get the last written track address in case of DVD+RW/DVD-RW
-		 * restricted overwrite since there is no such thing as track
-		 * number for these drives. */
-		media = brasero_medium_get_status (medium);
+		/* get the last written track address */
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_UINT64);
 
-		if (!BRASERO_MEDIUM_RANDOM_WRITABLE (media)) {
-			guint track_num;
+		BRASERO_BURN_LOG ("Start of last written track address == %lli", priv->session_start);
+		g_value_set_uint64 (value, priv->session_start);
+		brasero_track_tag_add (track,
+				       BRASERO_TRACK_MEDIUM_ADDRESS_START_TAG,
+				       value);
 
-			track_num = brasero_medium_get_track_num (medium);
+		value = g_new0 (GValue, 1);
+		g_value_init (value, G_TYPE_UINT64);
 
-			BRASERO_BURN_LOG ("Last written track num == %i", track_num);
-			brasero_track_disc_set_track_num (BRASERO_TRACK_DISC (track), track_num);
-		}
-		else {
-			GValue *value;
-
-			value = g_new0 (GValue, 1);
-			g_value_init (value, G_TYPE_UINT64);
-
-			BRASERO_BURN_LOG ("Start of last written track address == %lli", priv->session_start);
-			g_value_set_uint64 (value, priv->session_start);
-			brasero_track_tag_add (track,
-					       BRASERO_TRACK_MEDIUM_ADDRESS_START_TAG,
-					       value);
-
-			value = g_new0 (GValue, 1);
-			g_value_init (value, G_TYPE_UINT64);
-
-			BRASERO_BURN_LOG ("End of last written track address == %lli", priv->session_end);
-			g_value_set_uint64 (value, priv->session_end);
-			brasero_track_tag_add (track,
-					       BRASERO_TRACK_MEDIUM_ADDRESS_END_TAG,
-					       value);
-		}
+		BRASERO_BURN_LOG ("End of last written track address == %lli", priv->session_end);
+		g_value_set_uint64 (value, priv->session_end);
+		brasero_track_tag_add (track,
+				       BRASERO_TRACK_MEDIUM_ADDRESS_END_TAG,
+				       value);
 	}
 
 	result = brasero_burn_check_real (burn, track, error);
@@ -2263,6 +2331,20 @@ brasero_burn_record_session (BraseroBurn *burn,
 	return result;
 }
 
+/**
+ * brasero_burn_check:
+ * @burn: a #BraseroBurn
+ * @session: a #BraseroBurnSession
+ * @error: a #GError
+ *
+ * Checks the integrity of a medium according to the parameters
+ * set in @session. The medium must be inserted in the #BraseroDrive
+ * set as the source of a #BraseroTrackDisc track inserted in @session.
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult
 brasero_burn_check (BraseroBurn *self,
 		    BraseroBurnSession *session,
@@ -2328,11 +2410,9 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 {
 	gchar *toc = NULL;
 	gchar *image = NULL;
-	BraseroTrackImage *track;
 	GError *ret_error = NULL;
 	BraseroBurnResult result;
 	BraseroBurnPrivate *priv;
-	BraseroImageFormat format;
 	BraseroTrackType *output = NULL;
 
 	/* we can't create a proper list of tasks here since we don't know the
@@ -2342,33 +2422,27 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 
 	/* get the first possible format */
 	output = brasero_track_type_new ();
-	brasero_track_type_set_has_image (output);
+	result = brasero_burn_session_get_tmp_image_type_same_src_dest (priv->session, output);
 
-	format = BRASERO_IMAGE_FORMAT_CDRDAO;
-	for (; format != BRASERO_IMAGE_FORMAT_NONE; format >>= 1) {
-		brasero_track_type_set_image_format (output, format);
-		result = brasero_burn_session_output_supported (priv->session,
-								output);
-		if (result == BRASERO_BURN_OK)
-			break;
-	}
-	brasero_track_type_free (output);
-
-	if (format == BRASERO_IMAGE_FORMAT_NONE) {
+	if (result != BRASERO_BURN_OK) {
+		brasero_track_type_free (output);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
 			     _("No format for the temporary image could be found"));
-		return BRASERO_BURN_ERR;
+		return result;
 	}
 
-	/* get a new output. Also ask for both */
+	/* Save the flags for later */
 	brasero_burn_session_push_settings (priv->session);
+
+	/* get a new output. Also ask for both */
 	result = brasero_burn_session_get_tmp_image (priv->session,
-						     format,
+						     brasero_track_type_get_image_format (output),
 						     &image,
 						     &toc,
 						     &ret_error);
+
 	while (result != BRASERO_BURN_OK) {
 		gboolean is_temp;
 
@@ -2376,7 +2450,7 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 		||  (ret_error->code != BRASERO_BURN_ERROR_DISK_SPACE
 		&&   ret_error->code != BRASERO_BURN_ERROR_PERMISSION)) {
 			g_propagate_error (error, ret_error);
-			return result;
+			goto end;
 		}
 
 		/* That's an imager (outputs an image to the disc) so that means
@@ -2400,27 +2474,30 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 		g_error_free (ret_error);
 		ret_error = NULL;
 
-		if (result != BRASERO_BURN_OK)
-			return result;
+		if (result != BRASERO_BURN_OK) 
+			goto end;
 
 		/* retry */
 		result = brasero_burn_session_get_tmp_image (priv->session,
-							     format,
+							     brasero_track_type_get_image_format (output),
 							     &image,
 							     &toc,
 							     &ret_error);
 	}
 
 	/* some, like cdrdao, can't overwrite the files */
-	g_remove (image);
-	g_remove (toc);
+	if (image)
+		g_remove (image);
+
+	if (toc)
+		g_remove (toc);
 
 	result = brasero_burn_session_set_image_output_full (priv->session,
-							     format,
+							     brasero_track_type_get_image_format (output),
 							     image,
 							     toc);
 	if (result != BRASERO_BURN_OK)
-		return result;
+		goto end;
 
 	/* lock drive */
 	result = brasero_burn_lock_src_media (self, error);
@@ -2439,20 +2516,17 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 	if (result != BRASERO_BURN_OK)
 		goto end;
 
-	track = brasero_track_image_new ();
-	brasero_track_image_set_source (track, image, toc, format);
-	brasero_burn_session_add_track (priv->session, BRASERO_TRACK (track));
-
-	/* It's good practice to unref the track afterwards as we don't need it
-	 * anymore. BraseroBurnSession refs it. */
-	g_object_unref (track);
+	/* There should be a track at the top of the session stack
+	 * so no need to create a new one */
 
 end:
 	g_free (image);
 	g_free (toc);
 
-	brasero_burn_session_pop_settings (priv->session);
+	if (output)
+		brasero_track_type_free (output);
 
+	brasero_burn_session_pop_settings (priv->session);
 	return result;
 }
 
@@ -2530,6 +2604,19 @@ again:
 	return BRASERO_BURN_OK;
 }
 
+/**
+ * brasero_burn_record:
+ * @burn: a #BraseroBurn
+ * @session: a #BraseroBurnSession
+ * @error: a #GError
+ *
+ * Burns or creates a disc image according to the parameters
+ * set in @session.
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult 
 brasero_burn_record (BraseroBurn *burn,
 		     BraseroBurnSession *session,
@@ -2683,6 +2770,20 @@ brasero_burn_blank_real (BraseroBurn *burn, GError **error)
 	return result;
 }
 
+/**
+ * brasero_burn_blank:
+ * @burn: a #BraseroBurn
+ * @session: a #BraseroBurnSession
+ * @error: a #GError
+ *
+ * Blanks a medium according to the parameters
+ * set in @session. The medium must be inserted in the #BraseroDrive
+ * set with brasero_burn_session_set_burner ().
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult
 brasero_burn_blank (BraseroBurn *burn,
 		    BraseroBurnSession *session,
@@ -2750,6 +2851,19 @@ end:
 	return result;
 }
 
+/**
+ * brasero_burn_cancel:
+ * @burn: a #BraseroBurn
+ * @protect: a #gboolean
+ *
+ * Cancels any ongoing operation. If @protect is TRUE then
+ * cancellation will not take place for a "critical" task, a task whose interruption
+ * could damage the medium or the drive.
+ *
+ * Return value: a #BraseroBurnResult. The result of the operation. 
+ * BRASERO_BURN_OK if it was successful.
+ **/
+
 BraseroBurnResult
 brasero_burn_cancel (BraseroBurn *burn, gboolean protect)
 {
