@@ -43,18 +43,21 @@
 #include <gconf/gconf-client.h>
 
 #include "burn-basics.h"
-#include "brasero-track.h"
-#include "brasero-medium.h"
-#include "brasero-session.h"
-#include "brasero-drive.h"
-#include "brasero-volume.h"
-#include "brasero-burn-lib.h"
 #include "burn-plugin-manager.h"
 #include "brasero-medium-selection-priv.h"
+#include "brasero-session-helper.h"
 
 #include "brasero-dest-selection.h"
+
+#include "brasero-drive.h"
+#include "brasero-medium.h"
+#include "brasero-volume.h"
+
+#include "brasero-burn-lib.h"
+#include "brasero-tags.h"
+#include "brasero-track.h"
+#include "brasero-session.h"
 #include "brasero-session-cfg.h"
-#include "brasero-session-helper.h"
 
 typedef struct _BraseroDestSelectionPrivate BraseroDestSelectionPrivate;
 struct _BraseroDestSelectionPrivate
@@ -501,6 +504,29 @@ brasero_dest_selection_format_medium_string (BraseroMediumSelection *selection,
 	if (brasero_medium_get_status (medium) & BRASERO_MEDIUM_FILE) {
 		gchar *path;
 
+		input = brasero_track_type_new ();
+		brasero_burn_session_get_input_type (priv->session, input);
+
+		/* There should be a special name for image in video context */
+		if (brasero_track_type_get_has_stream (input)
+		&&  BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (input))) {
+			BraseroImageFormat format;
+
+			format = brasero_burn_session_get_output_format (priv->session);
+			if (format == BRASERO_IMAGE_FORMAT_CUE) {
+				g_free (medium_name);
+				if (brasero_burn_session_tag_lookup_int (priv->session, BRASERO_VCD_TYPE) == BRASERO_SVCD)
+					medium_name = g_strdup (_("SVCD image"));
+				else
+					medium_name = g_strdup (_("VCD image"));
+			}
+			else if (format == BRASERO_IMAGE_FORMAT_BIN) {
+				g_free (medium_name);
+				medium_name = g_strdup (_("Video DVD image"));
+			}
+		}
+		brasero_track_type_free (input);
+
 		/* get the set path for the image file */
 		path = brasero_dest_selection_get_output_path (BRASERO_DEST_SELECTION (selection));
 		if (!path)
