@@ -233,11 +233,13 @@ brasero_src_image_update (BraseroSrcImage *self)
 		/* Translators: %s is a path */
 		string = g_strdup_printf (_("\"%s\": loading"), path);
 		gtk_widget_set_tooltip_text (GTK_WIDGET (self), NULL);
+		g_free (path);
 		goto end;
 	}
 	else if (result != BRASERO_BURN_OK) {
 		/* Translators: %s is a path and image refers to a disc image */
 		string = g_strdup_printf (_("\"%s\": unknown disc image type"), path);
+		g_free (path);
 
 		error = brasero_status_get_error (status);
 		if (!error)
@@ -533,6 +535,7 @@ update_gconf:
 				 NULL);
 	g_object_unref (client);
 
+	g_object_unref (data);
 }
 
 static void
@@ -568,7 +571,7 @@ brasero_src_image_init (BraseroSrcImage *object)
 					 0,
 					 priv->cancel,
 					 brasero_src_image_set_parent_directory,
-					 object);
+					 g_object_ref (object));
 		g_object_unref (file);
 	}
 	g_free (uri);
