@@ -126,7 +126,7 @@ brasero_process_check_path (const gchar *name,
 	 * another symlink, then GIO didn't follow that other symlink. And in
 	 * the end it didn't work. So forbid all symlink. */
 	if (g_file_test (prog_path, G_FILE_TEST_IS_SYMLINK)) {
-		*error = g_strdup_printf (_("\"%s\" is a symlink pointing to another program. Use the target program instead"), name);
+		*error = g_strdup_printf (_("\"%s\" is a symbolic link pointing to another program. Use the target program instead"), name);
 		g_free (prog_path);
 		return BRASERO_BURN_ERR;
 	}
@@ -256,6 +256,7 @@ brasero_process_finished (BraseroProcess *self)
 	if (brasero_track_type_get_has_image (type)) {
 		gchar *toc = NULL;
 		gchar *image = NULL;
+		goffset blocks = 0;
 
 		track = BRASERO_TRACK (brasero_track_image_new ());
 		brasero_job_get_image_output (BRASERO_JOB (self),
@@ -269,6 +270,9 @@ brasero_process_finished (BraseroProcess *self)
 
 		g_free (image);
 		g_free (toc);
+
+		brasero_job_get_session_output_size (BRASERO_JOB (self), &blocks, NULL);
+		brasero_track_image_set_block_num (BRASERO_TRACK_IMAGE (track), blocks);
 	}
 	else if (brasero_track_type_get_has_stream (type)) {
 		gchar *uri = NULL;
@@ -597,7 +601,7 @@ brasero_process_start (BraseroJob *job, GError **error)
 	gboolean read_stdout;
 	/* that's to make sure programs are not translated */
 	gchar *envp [] = {	"LANG=C",
-				"LANGUAGE=C"
+				"LANGUAGE=C",
 				"LC_ALL=C",
 				NULL};
 
