@@ -104,6 +104,7 @@ brasero_medium_monitor_get_drive (BraseroMediumMonitor *monitor,
 	BraseroMediumMonitorPrivate *priv;
 
 	g_return_val_if_fail (monitor != NULL, NULL);
+	g_return_val_if_fail (device != NULL, NULL);
 	g_return_val_if_fail (BRASERO_IS_MEDIUM_MONITOR (monitor), NULL);
 
 	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (monitor);
@@ -236,41 +237,98 @@ brasero_medium_monitor_get_media (BraseroMediumMonitor *monitor,
 		if (!medium)
 			continue;
 
-		if ((type & BRASERO_MEDIA_TYPE_ANY_IN_BURNER)
-		&&  (brasero_drive_can_write (drive))) {
+		if ((type & BRASERO_MEDIA_TYPE_CD) == type
+		&& (brasero_medium_get_status (medium) & BRASERO_MEDIUM_CD)) {
+			/* If used alone, returns all CDs */
 			list = g_slist_prepend (list, medium);
 			g_object_ref (medium);
 			continue;
 		}
 
+		if ((type & BRASERO_MEDIA_TYPE_ANY_IN_BURNER)
+		&&  (brasero_drive_can_write (drive))) {
+			if ((type & BRASERO_MEDIA_TYPE_CD)) {
+				if ((brasero_medium_get_status (medium) & BRASERO_MEDIUM_CD)) {
+					list = g_slist_prepend (list, medium);
+					g_object_ref (medium);
+					continue;
+				}
+			}
+			else {
+				list = g_slist_prepend (list, medium);
+				g_object_ref (medium);
+				continue;
+			}
+			continue;
+		}
+
 		if ((type & BRASERO_MEDIA_TYPE_AUDIO)
 		&& !(brasero_medium_get_status (medium) & BRASERO_MEDIUM_FILE)
 		&&  (brasero_medium_get_status (medium) & BRASERO_MEDIUM_HAS_AUDIO)) {
-			list = g_slist_prepend (list, medium);
-			g_object_ref (medium);
+			if ((type & BRASERO_MEDIA_TYPE_CD)) {
+				if ((brasero_medium_get_status (medium) & BRASERO_MEDIUM_CD)) {
+					list = g_slist_prepend (list, medium);
+					g_object_ref (medium);
+					continue;
+				}
+			}
+			else {
+				list = g_slist_prepend (list, medium);
+				g_object_ref (medium);
+				continue;
+			}
 			continue;
 		}
 
 		if ((type & BRASERO_MEDIA_TYPE_DATA)
 		&& !(brasero_medium_get_status (medium) & BRASERO_MEDIUM_FILE)
 		&&  (brasero_medium_get_status (medium) & BRASERO_MEDIUM_HAS_DATA)) {
-			list = g_slist_prepend (list, medium);
-			g_object_ref (medium);
+			if ((type & BRASERO_MEDIA_TYPE_CD)) {
+				if ((brasero_medium_get_status (medium) & BRASERO_MEDIUM_CD)) {
+					list = g_slist_prepend (list, medium);
+					g_object_ref (medium);
+					continue;
+				}
+			}
+			else {
+				list = g_slist_prepend (list, medium);
+				g_object_ref (medium);
+				continue;
+			}
 			continue;
 		}
 
 		if (type & BRASERO_MEDIA_TYPE_WRITABLE) {
 			if (brasero_medium_can_be_written (medium)) {
-				list = g_slist_prepend (list, medium);
-				g_object_ref (medium);
-				continue;
+				if ((type & BRASERO_MEDIA_TYPE_CD)) {
+					if ((brasero_medium_get_status (medium) & BRASERO_MEDIUM_CD)) {
+						list = g_slist_prepend (list, medium);
+						g_object_ref (medium);
+						continue;
+					}
+				}
+				else {
+					list = g_slist_prepend (list, medium);
+					g_object_ref (medium);
+					continue;
+				}
 			}
 		}
 
 		if (type & BRASERO_MEDIA_TYPE_REWRITABLE) {
 			if (brasero_medium_can_be_rewritten (medium)) {
-				list = g_slist_prepend (list, medium);
-				g_object_ref (medium);
+				if ((type & BRASERO_MEDIA_TYPE_CD)) {
+					if ((brasero_medium_get_status (medium) & BRASERO_MEDIUM_CD)) {
+						list = g_slist_prepend (list, medium);
+						g_object_ref (medium);
+						continue;
+					}
+				}
+				else {
+					list = g_slist_prepend (list, medium);
+					g_object_ref (medium);
+					continue;
+				}
 				continue;
 			}
 		}
@@ -437,6 +495,9 @@ brasero_medium_monitor_volume_added_cb (GVolumeMonitor *monitor,
 	}
 
 	device = g_volume_get_identifier (gvolume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	if  (!device)
+		return;
+
 	brasero_medium_monitor_device_added (self, device, NULL);
 	g_free (device);
 }
@@ -553,6 +614,9 @@ brasero_medium_monitor_volume_removed_cb (GVolumeMonitor *monitor,
 	}
 
 	device = g_volume_get_identifier (gvolume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	if (!device)
+		return;
+
 	brasero_medium_monitor_device_removed (self, device, NULL);
 	g_free (device);
 }
@@ -627,6 +691,8 @@ brasero_medium_monitor_init (BraseroMediumMonitor *object)
 
 		gvolume = iter->data;
 		device = g_volume_get_identifier (gvolume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		if (!device)
+			continue;
 
 		/* make sure it isn't already in our list */
 		drive = brasero_medium_monitor_get_drive (object, device);
