@@ -364,6 +364,11 @@ brasero_io_return_result_idle (gpointer callback_data)
 		g_mutex_unlock (priv->lock);
 
 		data = result->callback_data;
+
+		/* This is to make sure the object lives
+		 * as long as we need it. */
+		g_object_ref (base->object);
+
 		if (result->uri || result->info || result->error)
 			result->base->callback (base->object,
 						result->error,
@@ -371,12 +376,14 @@ brasero_io_return_result_idle (gpointer callback_data)
 						result->info,
 						data? data->callback_data:NULL);
 
-		/* Else this is just to call destroy () for callback data */
+		/* call destroy () for callback data */
 		brasero_io_unref_result_callback_data (data,
 						       base->object,
 						       base->destroy,
 						       FALSE);
 
+		g_object_unref (base->object);
+
 		brasero_io_job_result_free (result);
 
 		g_mutex_lock (priv->lock);
@@ -589,15 +596,15 @@ brasero_io_mount_enclosing_volume (BraseroIO *self,
 	GMountOperation *operation;
 	BraseroIOMount mount = { NULL, };
 
-	/* FIXME: need a way to get a window for the operation */
-//	operation = gtk_mount_operation_new (GTK_WINDOW (brasero_app_get_default ()));
-	operation = NULL;
+	/* FIXME: need a way to get a parent window for the operation */
+	operation = gtk_mount_operation_new (NULL);
 	g_file_mount_enclosing_volume (file,
 				       G_MOUNT_MOUNT_NONE,
 				       operation,
 				       cancel,
 				       brasero_io_mount_enclosing_volume_cb,
 				       &mount);
+	g_object_unref (operation);
 
 	/* sleep and wait operation end */
 	while (!mount.finished && !g_cancellable_is_cancelled (cancel))
