@@ -587,7 +587,11 @@ foreach_tag (const GstTagList *list,
 		gst_tag_list_get_string (list, tag, &(self->composer));
 	}
 */	else if (!strcmp (tag, GST_TAG_ISRC)) {
-		gst_tag_list_get_int (list, tag, &(priv->info->isrc));
+		gchar *isrc = NULL;
+		gst_tag_list_get_string (list, tag, &isrc);
+
+		if (isrc)
+			priv->info->isrc = (int) g_ascii_strtoull (isrc, NULL, 10);
 	}
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID)) {
 		gst_tag_list_get_string (list, tag, &(priv->info->musicbrainz_id));
@@ -908,9 +912,12 @@ static void
 brasero_metadata_install_plugins_completed (BraseroMetadataGstDownload *download)
 {
 	GSList *iter;
+	GSList *next;
 
-	for (iter = download->objects; iter; iter = iter->next)
+	for (iter = download->objects; iter; iter = next) {
+		next = iter->next;
 		brasero_metadata_completed (BRASERO_METADATA (iter->data));
+	}
 }
 
 static void
