@@ -386,12 +386,14 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 	BraseroDeviceHandle *dev_handle;
 	BraseroVolFileHandle *handle;
 	BraseroBurnResult result;
+	BraseroDrive *burner;
+	BraseroMedium *medium;
 	BraseroVolFile *file;
 	BraseroTrack *track;
 	gchar buffer [2048];
 	BraseroVolSrc *vol;
 	goffset start_block;
-	gchar *device;
+	const gchar *device;
 
 	priv = BRASERO_CHECKSUM_FILES_PRIVATE (self);
 
@@ -407,9 +409,11 @@ brasero_checksum_files_merge_with_former_session (BraseroChecksumFiles *self,
 		return result;
 
 	/* try every file and make sure they are of the same type */
-	brasero_job_get_device (BRASERO_JOB (self), &device);
+	medium = NULL;
+	brasero_job_get_medium (BRASERO_JOB (self), &medium);
+	burner = brasero_medium_get_drive (medium);
+	device = brasero_drive_get_device (burner);
 	dev_handle = brasero_device_handle_open (device, FALSE, NULL);
-	g_free (device);
 
 	vol = brasero_volume_source_open_device_handle (dev_handle, error);
 	file = brasero_volume_get_file (vol,
@@ -524,16 +528,16 @@ brasero_checksum_files_create_checksum (BraseroChecksumFiles *self,
 	checksum_type = gconf_client_get_int (client, GCONF_KEY_CHECKSUM_TYPE, NULL);
 	g_object_unref (client);
 
-	if (checksum_type == BRASERO_CHECKSUM_NONE)
-		gchecksum_type = G_CHECKSUM_MD5;
-	else if (checksum_type & BRASERO_CHECKSUM_MD5_FILE)
+	if (checksum_type & BRASERO_CHECKSUM_MD5_FILE)
 		gchecksum_type = G_CHECKSUM_MD5;
 	else if (checksum_type & BRASERO_CHECKSUM_SHA1_FILE)
 		gchecksum_type = G_CHECKSUM_SHA1;
 	else if (checksum_type & BRASERO_CHECKSUM_SHA256_FILE)
 		gchecksum_type = G_CHECKSUM_SHA256;
-	else
+	else {
+		checksum_type = BRASERO_CHECKSUM_MD5_FILE;
 		gchecksum_type = G_CHECKSUM_MD5;
+	}
 
 	/* opens a file for the sums */
 	switch (gchecksum_type) {
