@@ -890,13 +890,13 @@ brasero_vob_build_video_bin (BraseroVob *vob,
 
 			aspect = g_value_get_int (value);
 			if (aspect == BRASERO_VIDEO_ASPECT_4_3) {
-				BRASERO_JOB_LOG (vob, "Setting ration 4:3");
+				BRASERO_JOB_LOG (vob, "Setting ratio 4:3");
 				g_object_set (encode,
 					      "aspect", 2,
 					      NULL);
 			}
 			else if (aspect == BRASERO_VIDEO_ASPECT_16_9) {
-				BRASERO_JOB_LOG (vob, "Setting ration 16:9");
+				BRASERO_JOB_LOG (vob, "Setting ratio 16:9");
 				g_object_set (encode,
 					      "aspect", 3,
 					      NULL);	
@@ -905,7 +905,7 @@ brasero_vob_build_video_bin (BraseroVob *vob,
 	}
 	else {
 		/* VCDs only support 4:3 */
-		BRASERO_JOB_LOG (vob, "Setting ration 4:3");
+		BRASERO_JOB_LOG (vob, "Setting ratio 4:3");
 		g_object_set (encode,
 			      "aspect", 2,
 			      NULL);
@@ -1127,8 +1127,8 @@ brasero_vob_start (BraseroJob *job,
 static BraseroBurnResult
 brasero_vob_clock_tick (BraseroJob *job)
 {
-	gint64 position = 0.0;
-	gint64 duration = 0.0;
+	gint64 position = 0;
+	gint64 duration = 0;
 	BraseroVobPrivate *priv;
 	GstFormat format = GST_FORMAT_TIME;
 
@@ -1136,14 +1136,15 @@ brasero_vob_clock_tick (BraseroJob *job)
 
 	gst_element_query_duration (priv->pipeline, &format, &duration);
 	gst_element_query_position (priv->pipeline, &format, &position);
-
-	if (duration <= 0.0 || position <= 0.0) {
+	if (duration <= 0 || position <= 0) {
 		format = GST_FORMAT_BYTES;
+		duration = 0;
+		position = 0;
 		gst_element_query_duration (priv->pipeline, &format, &duration);
 		gst_element_query_position (priv->pipeline, &format, &position);
 	}
 
-	if (duration > 0.0 && position > 0.0) {
+	if (duration > 0 && position > 0) {
 		gdouble progress;
 
 		progress = (gdouble) position / (gdouble) duration;
