@@ -33,6 +33,8 @@
 
 #include <gtk/gtk.h>
 
+#include <gconf/gconf-client.h>
+
 #include "brasero-misc.h"
 
 #include "eggtreemultidnd.h"
@@ -50,8 +52,6 @@
 #include "brasero-app.h"
 #include "brasero-project-manager.h"
 
-#include "burn-basics.h"
-
 #include "brasero-session-cfg.h"
 #include "brasero-tags.h"
 #include "brasero-track.h"
@@ -164,6 +164,9 @@ enum {
 	TARGET_URIS_LIST,
 };
 
+#define GCONF_COLUMN_KEY				"/apps/brasero/display/data_column"
+#define GCONF_COLUMN_ORDER_KEY		"/apps/brasero/display/data_sort_order"
+
 static GtkTargetEntry ntables_cd [] = {
 	{BRASERO_DND_TARGET_DATA_TRACK_REFERENCE_LIST, GTK_TARGET_SAME_WIDGET, TREE_MODEL_ROW},
 	{"text/uri-list", 0, TARGET_URIS_LIST}
@@ -192,7 +195,7 @@ G_DEFINE_TYPE_WITH_CODE (BraseroDataDisc,
 
 #define BRASERO_DATA_DISC_MEDIUM		"brasero-data-disc-medium"
 #define BRASERO_DATA_DISC_MERGE_ID		"brasero-data-disc-merge-id"
-#define BRASERO_MEDIUM_GET_UDI(medium)		(brasero_drive_get_udi (brasero_medium_get_drive (medium)))
+#define BRASERO_MEDIUM_GET_UDI(medium)		(brasero_drive_get_device (brasero_medium_get_drive (medium)))
 
 BraseroMedium *
 brasero_data_disc_get_loaded_medium (BraseroDataDisc *self)
@@ -588,6 +591,19 @@ brasero_data_disc_project_loaded_cb (BraseroTrackDataCfg *project,
 	}
 }
 
+static gboolean
+brasero_data_disc_launch_image (gpointer data)
+{
+	gchar *uri = data;
+	GtkWidget *manager;
+
+	manager = brasero_app_get_project_manager (brasero_app_get_default ());
+	brasero_project_manager_iso (BRASERO_PROJECT_MANAGER (manager), uri);
+	g_free (uri);
+
+	return FALSE;
+}
+
 static BraseroBurnResult
 brasero_data_disc_image_uri_cb (BraseroTrackDataCfg *vfs,
 				const gchar *uri,
@@ -598,7 +614,6 @@ brasero_data_disc_image_uri_cb (BraseroTrackDataCfg *vfs,
 	gchar *string;
 	GtkWidget *button;
 	GtkWidget *dialog;
-	GtkWidget *manager;
 	BraseroDataDiscPrivate *priv;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
@@ -633,10 +648,8 @@ brasero_data_disc_image_uri_cb (BraseroTrackDataCfg *vfs,
 	if (answer != GTK_RESPONSE_YES)
 		return BRASERO_BURN_OK;
 
-	/* Tell project manager to switch. First function to avoid warnings */
-	brasero_track_data_cfg_reset (priv->project);
-	manager = brasero_app_get_project_manager (brasero_app_get_default ());
-	brasero_project_manager_iso (BRASERO_PROJECT_MANAGER (manager), uri);
+	/* Tell project manager to switch. */
+	g_idle_add (brasero_data_disc_launch_image, g_strdup (uri));
 
 	return BRASERO_BURN_CANCEL;
 }
@@ -768,13 +781,13 @@ brasero_data_disc_name_collision_cb (BraseroTrackDataCfg *project,
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
 
 	if (priv->always_replace)
-		return TRUE;
-
-	if (priv->never_replace)
 		return FALSE;
 
+	if (priv->never_replace)
+		return TRUE;
+
 	/* Translators: %s is the name of the file */
-	string = g_strdup_printf (_("Do you really want to replace \"%s\"?"), name);
+	string = g_strdup_printf (_("Do you want to replace \"%s\"?"), name);
 	dialog = brasero_app_dialog (brasero_app_get_default (),
 				     string,
 				     GTK_BUTTONS_NONE,
@@ -782,7 +795,7 @@ brasero_data_disc_name_collision_cb (BraseroTrackDataCfg *project,
 	g_free (string);
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("A file with the same name is about to be added into the same directory and will replace it if you accept."));
+	                                          _("A file with this name already exists in the folder.  Replacing it will overwrite its content (on the disc only)."));
 
 	/* Translators: Keep means we're keeping the files that already existed
 	 * in the project.
@@ -960,7 +973,7 @@ brasero_disc_disc_session_import_response_cb (GtkButton *button,
 	g_signal_handlers_unblock_by_func (action, brasero_data_disc_import_session_cb, self);
 }
 
-static void
+static GtkAction *
 brasero_data_disc_import_button_new (BraseroDataDisc *self,
 				     BraseroMedium *medium)
 {
@@ -977,7 +990,7 @@ brasero_data_disc_import_button_new (BraseroDataDisc *self,
 	priv = BRASERO_DATA_DISC_PRIVATE (self);
 
 	if (!priv->manager)
-		return;
+		return NULL;
 
 	action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
 
@@ -1011,7 +1024,7 @@ brasero_data_disc_import_button_new (BraseroDataDisc *self,
 	action = gtk_action_group_get_action (priv->import_group, action_name);
 	if (!action) {
 		g_free (action_name);
-		return;
+		return NULL;
 	}
 
 	g_object_ref (medium);
@@ -1051,6 +1064,32 @@ brasero_data_disc_import_button_new (BraseroDataDisc *self,
 
 	g_free (description);
 	g_free (action_name);
+	return action;
+}
+
+static void
+brasero_data_disc_remove_available_medium (BraseroDataDisc *self,
+                                           BraseroMedium *medium)
+{
+	int merge_id;
+	GtkAction *action;
+	gchar *action_name;
+	BraseroDataDiscPrivate *priv;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (self);
+
+	action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
+	action = gtk_action_group_get_action (priv->import_group, action_name);
+	g_free (action_name);
+
+	brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
+
+	merge_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MERGE_ID));
+	gtk_ui_manager_remove_ui (priv->manager, merge_id);
+	gtk_action_group_remove_action (priv->import_group, action);
+
+	/* unref it since we reffed it when it was associated with the action */
+	g_object_unref (medium);
 }
 
 static void
@@ -1106,24 +1145,8 @@ brasero_data_disc_session_available_cb (BraseroTrackDataCfg *session,
 				  G_CALLBACK (brasero_disc_disc_session_import_response_cb),
 				  self);
 	}
-	else {
-		int merge_id;
-		GtkAction *action;
-		gchar *action_name;
-
-		action_name = g_strdup_printf ("Import_%s", BRASERO_MEDIUM_GET_UDI (medium));
-		action = gtk_action_group_get_action (priv->import_group, action_name);
-		g_free (action_name);
-
-		brasero_notify_message_remove (BRASERO_NOTIFY (priv->message), BRASERO_NOTIFY_CONTEXT_MULTISESSION);
-
-		merge_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MERGE_ID));
-		gtk_ui_manager_remove_ui (priv->manager, merge_id);
-		gtk_action_group_remove_action (priv->import_group, action);
-
-		/* unref it since we reffed it when it was associated with the action */
-		g_object_unref (medium);
-	}
+	else
+		brasero_data_disc_remove_available_medium (self, medium);
 }
 
 static void
@@ -1279,18 +1302,61 @@ brasero_data_disc_message_response_cb (BraseroDiscMessage *message,
 	brasero_data_disc_clear (BRASERO_DISC (self));
 }
 
+static 
+brasero_data_disc_sort_column_changed (GtkTreeSortable *sortable,
+                                       BraseroDataDisc *disc)
+{
+	GtkSortType sort_order;
+	GConfClient *client;
+	gint sort_column;
+
+	gtk_tree_sortable_get_sort_column_id (sortable, &sort_column, &sort_order);
+
+	client = gconf_client_get_default ();
+	gconf_client_set_int (client,
+	                      GCONF_COLUMN_KEY,
+	                      sort_column,
+	                      NULL);
+	gconf_client_set_int (client,
+	                      GCONF_COLUMN_ORDER_KEY,
+	                      sort_order,
+	                      NULL);
+}
+
 static BraseroDiscResult
 brasero_data_disc_set_track (BraseroDataDisc *disc,
 			     BraseroTrackDataCfg *track)
 {
+	BraseroMedium *loaded_medium;
 	BraseroDataDiscPrivate *priv;
 	BraseroStatus *status;
 	GtkWidget *message;
+	GConfClient *client;
+	gint sort_column;
+	gint sort_order;
 
 	priv = BRASERO_DATA_DISC_PRIVATE (disc);
 
 	priv->project = g_object_ref (track);
 
+	client = gconf_client_get_default ();
+	sort_column = gconf_client_get_int (client,
+	                                    GCONF_COLUMN_KEY,
+	                                    NULL);
+	sort_order = gconf_client_get_int (client,
+	                                   GCONF_COLUMN_ORDER_KEY,
+	                                   NULL);
+
+	if ((sort_column == BRASERO_DATA_TREE_MODEL_SIZE
+	||   sort_column == BRASERO_DATA_TREE_MODEL_MIME_DESC
+	||   sort_column == BRASERO_DATA_TREE_MODEL_NAME)
+	&& (sort_order == GTK_SORT_ASCENDING
+	||  sort_order == GTK_SORT_DESCENDING))
+		gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (track),
+						      sort_column,
+						      sort_order);
+	g_object_unref (client);
+
 	/* filtered files */
 	priv->filter = brasero_file_filtered_new (priv->project);
 	brasero_file_filtered_set_right_button_group (BRASERO_FILE_FILTERED (priv->filter), priv->button_size);
@@ -1301,7 +1367,12 @@ brasero_data_disc_set_track (BraseroDataDisc *disc,
 	gtk_widget_show (priv->filter);
 	gtk_box_pack_end (GTK_BOX (disc), priv->filter, FALSE, TRUE, 0);
 
+	/* Show all actions */
+	if (!gtk_action_group_get_visible (priv->disc_group))
+		gtk_action_group_set_visible (priv->disc_group, TRUE);
+
 	/* Now let's take care of all the available sessions */
+	loaded_medium = brasero_track_data_cfg_get_current_medium (track);
 	if (!priv->import_group) {
 		GSList *iter;
 		GSList *list;
@@ -1315,9 +1386,15 @@ brasero_data_disc_set_track (BraseroDataDisc *disc,
 		list = brasero_track_data_cfg_get_available_media (priv->project);
 		for (iter = list; iter; iter = iter->next) {
 			BraseroMedium *medium;
+			GtkAction *action;
 
 			medium = iter->data;
-			brasero_data_disc_import_button_new (BRASERO_DATA_DISC (disc), medium);
+			action = brasero_data_disc_import_button_new (BRASERO_DATA_DISC (disc), medium);
+			if (!action)
+				continue;
+
+			if (medium == loaded_medium)
+				gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 		}
 		g_slist_foreach (list, (GFunc) g_object_unref, NULL);
 		g_slist_free (list);
@@ -1325,6 +1402,11 @@ brasero_data_disc_set_track (BraseroDataDisc *disc,
 	else
 		gtk_action_group_set_visible (priv->import_group, TRUE);
 
+	g_signal_connect (track,
+	                  "sort-column-changed",
+	                  G_CALLBACK (brasero_data_disc_sort_column_changed),
+	                  disc);
+
 	g_signal_connect (track,
 			  "2G-file",
 			  G_CALLBACK (brasero_data_disc_2G_file_cb),
@@ -1450,28 +1532,51 @@ brasero_data_disc_unset_track (BraseroDataDisc *disc)
 		priv->filter = NULL;
 	}
 
-	/* Unload session */
-	if (brasero_track_data_cfg_get_current_medium (BRASERO_TRACK_DATA_CFG (priv->project)))
-		brasero_track_data_cfg_unload_current_medium (BRASERO_TRACK_DATA_CFG (priv->project));
-
 	if (priv->size_changed_id) {
 		g_source_remove (priv->size_changed_id);
 		priv->size_changed_id = 0;
 	}
 
-	/* Hide all toggle actions for session importing */
-	if (gtk_action_group_get_visible (priv->import_group))
-		gtk_action_group_set_visible (priv->import_group, FALSE);
-
+	/* Hide all actions */
 	if (gtk_action_group_get_visible (priv->disc_group))
 		gtk_action_group_set_visible (priv->disc_group, FALSE);
 
+	/* Remove each button for every available session that can be imported */
+	if (priv->import_group) {
+		GList *actions;
+		GList *iter;
+
+		actions = gtk_action_group_list_actions (priv->import_group);
+		for (iter = actions; iter; iter = iter->next) {
+			BraseroMedium *medium;
+			GtkAction *action;
+			int merge_id;
+
+			action = iter->data;
+
+			/* We reffed the medium associated with the action */
+			medium = g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MEDIUM);
+			g_object_unref (medium);
+
+			merge_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (action), BRASERO_DATA_DISC_MERGE_ID));
+			gtk_ui_manager_remove_ui (priv->manager, merge_id);
+		}
+		g_list_free (actions);
+
+		gtk_ui_manager_remove_action_group (priv->manager,  priv->import_group);
+		g_object_unref (priv->import_group);
+		priv->import_group = NULL;
+	}
+
 	if (priv->load_errors) {
 		g_slist_foreach (priv->load_errors, (GFunc) g_free , NULL);
 		g_slist_free (priv->load_errors);
 		priv->load_errors = NULL;
 	}
 
+	g_signal_handlers_disconnect_by_func (priv->project,
+					      brasero_data_disc_sort_column_changed,
+					      disc);
 	g_signal_handlers_disconnect_by_func (priv->project,
 					      brasero_data_disc_2G_file_cb,
 					      disc);
@@ -1558,6 +1663,9 @@ brasero_data_disc_set_session_contents (BraseroDisc *self,
 						NULL);
 		brasero_data_disc_set_track (BRASERO_DATA_DISC (self),
 					     BRASERO_TRACK_DATA_CFG (data_track));
+
+		/* NOTE: that track was reffed in brasero_data_disc_set_track () */
+		g_object_unref (data_track);
 	}
 	else for (; tracks; tracks = tracks->next) {
 		BraseroTrack *track;
@@ -2301,6 +2409,11 @@ brasero_data_disc_finalize (GObject *object)
 
 	priv = BRASERO_DATA_DISC_PRIVATE (object);
 
+	if (priv->project) {
+		g_object_unref (priv->project);
+		priv->project = NULL;
+	}
+
 	if (priv->button_size) {
 		g_object_unref (priv->button_size);
 		priv->button_size = NULL;
@@ -2325,11 +2438,26 @@ brasero_data_disc_finalize (GObject *object)
 	G_OBJECT_CLASS (brasero_data_disc_parent_class)->finalize (object);
 }
 
+static gboolean
+brasero_data_disc_is_empty (BraseroDisc *disc)
+{
+	BraseroDataDiscPrivate *priv;
+	GtkTreeModel *model;
+
+	priv = BRASERO_DATA_DISC_PRIVATE (disc);
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree));
+	if(!model)
+		return FALSE;
+
+	return gtk_tree_model_iter_n_children (model, NULL) != 0;
+}
+
 static void
 brasero_data_disc_iface_disc_init (BraseroDiscIface *iface)
 {
 	iface->add_uri = brasero_data_disc_add_uri;
 	iface->delete_selected = brasero_data_disc_delete_selected;
+	iface->is_empty = brasero_data_disc_is_empty;
 	iface->clear = brasero_data_disc_clear;
 	iface->set_session_contents = brasero_data_disc_set_session_contents;
 	iface->get_selected_uri = brasero_data_disc_get_selected_uri;
