@@ -7,7 +7,8 @@
  * Based on gedit code (gedit/gedit-plugin-manager.c) by: 
  * 	- Paolo Maggi <paolo@gnome.org>
  *
- * Brasero is free software; you can redistribute it and/or modify
+ * Libbrasero-media is free software; you can redistribute it and/or modify
+fy
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
@@ -34,12 +35,13 @@
 
 #include <gconf/gconf-client.h>
 
+#include "brasero-misc.h"
+
 #include "brasero-plugin-manager-ui.h"
-#include "brasero-utils.h"
-#include "burn-plugin.h"
-#include "burn-caps.h"
-#include "burn-plugin-private.h"
-#include "burn-plugin-manager.h"
+
+#include "brasero-plugin.h"
+#include "brasero-plugin-information.h"
+#include "brasero-burn-lib.h"
 #include "brasero-plugin-option.h"
 
 typedef enum {
@@ -344,7 +346,6 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 {
 	BraseroPluginManagerUIPrivate *priv;
 	BraseroPlugin *plugin;
-	BraseroBurnCaps *caps;
 	const GSList *plugins;
 	GtkListStore *model;
 	GtkTreeIter iter;
@@ -352,17 +353,13 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 	priv = BRASERO_PLUGIN_MANAGER_UI_GET_PRIVATE (pm);
 	plugins = priv->plugins;
 
-	caps = brasero_burn_caps_get_default ();
-
 	model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
 
 	for (; plugins; plugins = plugins->next) {
 		plugin = plugins->data;
 
-		if (brasero_plugin_can_burn (plugin) == BRASERO_BURN_OK
-		||  brasero_plugin_can_convert (plugin) == BRASERO_BURN_OK
-		||  brasero_plugin_can_image (plugin) == BRASERO_BURN_OK
-		||  brasero_plugin_get_gtype (plugin) == G_TYPE_NONE)
+		if (brasero_plugin_get_gtype (plugin) == G_TYPE_NONE
+		||  brasero_plugin_get_compulsory (plugin))
 			continue;
 
 		gtk_list_store_append (model, &iter);
@@ -372,7 +369,6 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 				    PLUGIN_COLUMN, plugin,
 				    -1);
 	}
-	g_object_unref (caps);
 
 	plugin = NULL;
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter))
@@ -916,7 +912,6 @@ brasero_plugin_manager_ui_init (BraseroPluginManagerUI *pm)
 	GtkWidget *viewport;
 	GtkWidget *alignment;
 	GtkWidget *vbuttonbox;
-	BraseroPluginManager *manager;
 	BraseroPluginManagerUIPrivate *priv;
 
 	priv = BRASERO_PLUGIN_MANAGER_UI_GET_PRIVATE (pm);
@@ -981,8 +976,7 @@ brasero_plugin_manager_ui_init (BraseroPluginManagerUI *pm)
 	plugin_manager_ui_construct_tree (pm);
 
 	/* get the list of available plugins (or installed) */
-	manager = brasero_plugin_manager_get_default ();
-	priv->plugins = brasero_plugin_manager_get_plugins_list (manager);
+	priv->plugins = brasero_burn_library_get_plugins_list ();
 
 	if (!priv->plugins){
 		gtk_widget_set_sensitive (priv->about_button, FALSE);
@@ -1001,6 +995,7 @@ brasero_plugin_manager_ui_finalize (GObject *object)
 	priv = BRASERO_PLUGIN_MANAGER_UI_GET_PRIVATE (pm);
 
 	if (priv->plugins) {
+		g_slist_foreach (priv->plugins, (GFunc) g_object_unref, NULL);
 		g_slist_free (priv->plugins);
 		priv->plugins = NULL;
 	}
