@@ -45,11 +45,11 @@
 
 #include "brasero-project-manager.h"
 #include "brasero-multi-dnd.h"
-#include "brasero-session.h"
 #include "brasero-utils.h"
 #include "brasero-app.h"
-#include "burn-debug.h"
-#include "burn.h"
+
+#include "brasero-burn-lib.h"
+#include "brasero-session.h"
 
 #include "eggsmclient.h"
 
@@ -68,9 +68,6 @@ gint disc_blank;
 gint disc_check;
 gint open_ncb;
 gint parent_window;
-gint debug;
-
-#define BRASERO_CONF_DIR "/apps/brasero"
 
 static const GOptionEntry options [] = {
 	{ "project", 'p', 0, G_OPTION_ARG_STRING, &project_uri,
@@ -137,10 +134,6 @@ static const GOptionEntry options [] = {
 	 * parent as if brasero was a dialog for the parent application */
 	  N_("The XID of the parent window"), NULL },
 
-	{ "debug", 'g', 0, G_OPTION_ARG_NONE, &debug,
-	  N_("Display debug statements on stdout"),
-	  NULL },
-
 	{ G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &files,
 	  NULL, NULL }, /* collects file arguments */
 
@@ -211,7 +204,7 @@ brasero_handle_burn_uri (BraseroApp *app,
 			string = g_strdup (_("An internal error occured"));
 
 		brasero_app_alert (app,
-				   _("Error while loading the project."),
+				   _("Error while loading the project"),
 				   string,
 				   GTK_MESSAGE_ERROR);
 
@@ -237,12 +230,11 @@ brasero_handle_burn_uri (BraseroApp *app,
 			string = g_strdup (_("An internal error occured"));
 
 		brasero_app_alert (app,
-				   _("Error while loading the project."),
+				   _("Error while loading the project"),
 				   string,
 				   GTK_MESSAGE_ERROR);
 
 		g_free (string);
-		g_object_unref (file);
 
 		g_slist_foreach (list, (GFunc) g_free, NULL);
 		g_slist_free (list);
@@ -268,43 +260,6 @@ brasero_handle_burn_uri (BraseroApp *app,
 	return;
 }
 
-static gboolean
-brasero_app_open_project (BraseroApp *app,
-			  const gchar *path,
-			  gboolean playlist,
-			  gboolean burn)
-{
-	BraseroDiscTrack *track = NULL;
-	BraseroProjectType type;
-	GtkWidget *manager;
-	GFile *file;
-	gchar *uri;
-
-	brasero_app_create_mainwin (app);
-
-	file = g_file_new_for_commandline_arg (path);
-	uri = g_file_get_uri (file);
-	g_object_unref (file);
-
-#ifdef BUILD_PLAYLIST
-
-	if (playlist)
-		type = brasero_project_open_audio_playlist_project (uri, &track, TRUE);
-	else
-
-#endif
-		
-		type = brasero_project_open_project_xml (uri, &track, TRUE);
-
-	if (type == BRASERO_PROJECT_TYPE_INVALID)
-		return FALSE;
-
-	manager = brasero_app_get_project_manager (app);
-	brasero_project_manager_open_project (BRASERO_PROJECT_MANAGER (manager), track, uri, burn);
-
-	return TRUE;
-}
-
 static void
 brasero_app_parse_options (BraseroApp *app)
 {
@@ -355,10 +310,10 @@ brasero_app_parse_options (BraseroApp *app)
 		brasero_project_manager_empty (BRASERO_PROJECT_MANAGER (manager));
 	}
 	else if (project_uri) {
-		brasero_app_open_project (app, project_uri, FALSE, FALSE);
+		brasero_app_open_project (app, project_uri, FALSE, TRUE, FALSE);
 	}
 	else if (burn_project_uri) {
-		brasero_app_open_project (app, burn_project_uri, FALSE, TRUE);
+		brasero_app_open_project (app, burn_project_uri, FALSE, TRUE, TRUE);
 
 		if (g_remove (burn_project_uri) != 0) {
 			gchar *path;
@@ -373,7 +328,7 @@ brasero_app_parse_options (BraseroApp *app)
 #ifdef BUILD_PLAYLIST
 
 	else if (playlist_uri) {
-		brasero_app_open_project (app, playlist_uri, TRUE, FALSE);
+		brasero_app_open_project (app, playlist_uri, TRUE, TRUE, FALSE);
 	}
 
 #endif
@@ -451,21 +406,13 @@ brasero_app_parse_options (BraseroApp *app)
 		return;
 	}
 	else if (files) {
+		if (g_strv_length (files) == 1
+		&&  brasero_app_open_uri (app, files [0], FALSE))
+			return;
+
 		brasero_app_create_mainwin (app);
 		manager = brasero_app_get_project_manager (app);
-
-		if (g_strv_length (files) == 1) {
-			BraseroProjectType type;
-
-			type = brasero_project_manager_open_uri (BRASERO_PROJECT_MANAGER (manager), files [0]);
-
-			/* Fallback if it hasn't got a suitable URI */
-			if (type == BRASERO_PROJECT_TYPE_INVALID) {
-				BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files, FALSE);
-			}
-		}
-		else
-			BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files, FALSE);
+		BRASERO_PROJECT_OPEN_LIST (manager, brasero_project_manager_data, files, FALSE);
 	}
 	else {
 		brasero_app_create_mainwin (app);
@@ -491,10 +438,8 @@ brasero_app_get_default (void)
 int
 main (int argc, char **argv)
 {
-	GConfClient *client;
 	GOptionContext *context;
 
-
 #ifdef ENABLE_NLS
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -513,6 +458,7 @@ main (int argc, char **argv)
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
 	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 	g_option_context_add_group (context, brasero_media_get_option_group ());
+	g_option_context_add_group (context, brasero_burn_library_get_option_group ());
 	g_option_context_add_group (context, gst_init_get_option_group ());
 	if (g_option_context_parse (context, &argc, &argv, NULL) == FALSE) {
 		g_print (_("Please type %s --help to see all available options\n"), argv [0]);
@@ -522,22 +468,18 @@ main (int argc, char **argv)
 
 	g_option_context_free (context);
 
-	gst_init (&argc, &argv);
-
-	/* This is for missing codec automatic install */
+	/* REMINDER: this is done in burn library now */
+/*	gst_init (&argc, &argv);
 	gst_pb_utils_init ();
-
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client,
 			      BRASERO_CONF_DIR,
 			      GCONF_CLIENT_PRELOAD_NONE,
 			      NULL);
-
-	brasero_burn_set_debug (debug);
-	brasero_burn_library_init ();
+*/
+	brasero_burn_library_start (&argc, &argv);
 
 	brasero_enable_multi_DND ();
-	brasero_utils_init ();
 
 	current_app = brasero_app_new ();
 	if (current_app == NULL)
@@ -546,10 +488,7 @@ main (int argc, char **argv)
 	brasero_app_parse_options (current_app);
 	current_app = NULL;
 
-	brasero_burn_library_shutdown ();
-
-	gconf_client_remove_dir (client, BRASERO_CONF_DIR, NULL);
-	g_object_unref (client);
+	brasero_burn_library_stop ();
 
 	gst_deinit ();
 
